"""Initial revision

Revision ID: 3a302d32b0ad
Revises:
Create Date: 2024-05-07 14:00:13.618993

"""

import time
from typing import Sequence, Union

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import mysql

import clowm.db.types
from clowm.db.base_class import uuid7

# revision identifiers, used by Alembic.
revision: str = "3a302d32b0ad"
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "role",
        sa.Column("role_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("name", sa.String(length=64), nullable=False),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("role_id"),
    )
    op.create_index(op.f("ix_role_role_id"), "role", ["role_id"], unique=True)
    op.create_table(
        "user",
        sa.Column("uid", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("lifescience_id", sa.String(length=64), nullable=True),
        sa.Column("display_name", sa.String(length=256), nullable=False),
        sa.Column("email", sa.String(length=256), nullable=True),
        sa.Column("aruna_token", sa.String(length=128), nullable=True),
        sa.Column("initialized", sa.Boolean(), nullable=False),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("uid"),
    )
    op.create_index(op.f("ix_user_lifescience_id"), "user", ["lifescience_id"], unique=True)
    op.create_index(op.f("ix_user_uid"), "user", ["uid"], unique=True)
    op.create_table(
        "workflow_mode",
        sa.Column("mode_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("schema_path", sa.String(length=256), nullable=False),
        sa.Column("entrypoint", sa.String(length=256), nullable=False),
        sa.Column("name", sa.String(length=128), nullable=False),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("mode_id"),
    )
    op.create_index(op.f("ix_workflow_mode_mode_id"), "workflow_mode", ["mode_id"], unique=True)
    op.create_table(
        "bucket",
        sa.Column("name", sa.String(length=63), nullable=False),
        sa.Column("description", mysql.TEXT(), nullable=False),
        sa.Column("public", sa.Boolean(), nullable=False),
        sa.Column("owner_id", clowm.db.types.SQLUUID(length=16), nullable=True),
        sa.Column("size_limit", mysql.INTEGER(unsigned=True), nullable=True),
        sa.Column("object_limit", mysql.INTEGER(unsigned=True), nullable=True),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["owner_id"], ["user.uid"], ondelete="SET NULL"),
        sa.PrimaryKeyConstraint("name"),
    )
    op.create_index(op.f("ix_bucket_name"), "bucket", ["name"], unique=True)
    op.create_table(
        "resource",
        sa.Column("resource_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("name", sa.String(length=32), nullable=False),
        sa.Column("short_description", sa.String(length=264), nullable=False),
        sa.Column("source", sa.String(length=264), nullable=False),
        sa.Column("private", sa.Boolean(), nullable=False),
        sa.Column("maintainer_id", clowm.db.types.SQLUUID(length=16), nullable=True),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["maintainer_id"], ["user.uid"], ondelete="SET NULL"),
        sa.PrimaryKeyConstraint("resource_id"),
        sa.UniqueConstraint("name"),
    )
    op.create_index(op.f("ix_resource_resource_id"), "resource", ["resource_id"], unique=True)
    op.create_table(
        "user_role_mapping",
        sa.Column("uid", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("role_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["role_id"], ["role.role_id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["uid"], ["user.uid"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("uid", "role_id"),
    )
    op.create_table(
        "workflow",
        sa.Column("workflow_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("name", sa.String(length=63), nullable=False),
        sa.Column("repository_url", sa.String(length=256), nullable=False),
        sa.Column("short_description", mysql.TEXT(), nullable=False),
        sa.Column("credentials_token", sa.String(length=128), nullable=True),
        sa.Column("developer_id", clowm.db.types.SQLUUID(length=16), nullable=True),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["developer_id"], ["user.uid"], ondelete="SET NULL"),
        sa.PrimaryKeyConstraint("workflow_id"),
        sa.UniqueConstraint("name"),
    )
    op.create_index(op.f("ix_workflow_workflow_id"), "workflow", ["workflow_id"], unique=True)
    op.create_table(
        "bucketpermission",
        sa.Column("uid", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("bucket_name", sa.String(length=63), nullable=False),
        sa.Column("from", sa.Integer(), nullable=True),
        sa.Column("to", sa.Integer(), nullable=True),
        sa.Column("file_prefix", sa.String(length=512), nullable=True),
        sa.Column("permissions", mysql.ENUM("READ", "WRITE", "READWRITE"), nullable=False),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["bucket_name"], ["bucket.name"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["uid"], ["user.uid"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("uid", "bucket_name"),
    )
    op.create_table(
        "resource_version",
        sa.Column("resource_version_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("release", sa.String(length=32), nullable=False),
        sa.Column(
            "status",
            mysql.ENUM(
                "RESOURCE_REQUESTED",
                "WAIT_FOR_REVIEW",
                "DENIED",
                "APPROVED",
                "SYNC_REQUESTED",
                "SYNCHRONIZING",
                "SYNC_ERROR",
                "SYNCHRONIZED",
                "SETTING_LATEST",
                "LATEST",
                "CLUSTER_DELETING",
                "CLUSTER_DELETE_ERROR",
                "S3_DELETING",
                "S3_DELETE_ERROR",
                "S3_DELETED",
            ),
            nullable=False,
        ),
        sa.Column("sync_finished_timestamp", sa.Integer(), nullable=True),
        sa.Column("sync_slurm_job_id", sa.Integer(), nullable=True),
        sa.Column("cluster_deleted_timestamp", sa.Integer(), nullable=True),
        sa.Column("cluster_delete_slurm_job_id", sa.Integer(), nullable=True),
        sa.Column("s3_deleted_timestamp", sa.Integer(), nullable=True),
        sa.Column("set_latest_timestamp", sa.Integer(), nullable=True),
        sa.Column("set_latest_slurm_job_id", sa.Integer(), nullable=True),
        sa.Column("remove_latest_timestamp", sa.Integer(), nullable=True),
        sa.Column("resource_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("synchronization_request_uid", clowm.db.types.SQLUUID(length=16), nullable=True),
        sa.Column("synchronization_request_description", sa.String(length=512), nullable=True),
        sa.Column("last_used_timestamp", sa.Integer(), nullable=True),
        sa.Column("times_used", sa.Integer(), nullable=False),
        sa.Column("previous_resource_version_id", clowm.db.types.SQLUUID(length=16), nullable=True),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["previous_resource_version_id"], ["resource_version.resource_version_id"], ondelete="SET NULL"
        ),
        sa.ForeignKeyConstraint(["resource_id"], ["resource.resource_id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["synchronization_request_uid"], ["user.uid"], ondelete="SET NULL"),
        sa.PrimaryKeyConstraint("resource_version_id"),
    )
    op.create_index(
        op.f("ix_resource_version_resource_version_id"), "resource_version", ["resource_version_id"], unique=True
    )
    op.create_table(
        "workflow_version",
        sa.Column("git_commit_hash", sa.String(length=40), nullable=False),
        sa.Column("version", sa.String(length=10), nullable=False),
        sa.Column("status", mysql.ENUM("CREATED", "DENIED", "PUBLISHED", "DEPRECATED"), nullable=False),
        sa.Column("icon_slug", sa.String(length=48), nullable=True),
        sa.Column("parameter_extension", sa.JSON(), nullable=True),
        sa.Column("workflow_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("previous_version_hash", sa.String(length=40), nullable=True),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["previous_version_hash"], ["workflow_version.git_commit_hash"], ondelete="SET NULL"),
        sa.ForeignKeyConstraint(["workflow_id"], ["workflow.workflow_id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("git_commit_hash"),
    )
    op.create_index(op.f("ix_workflow_version_git_commit_hash"), "workflow_version", ["git_commit_hash"], unique=True)
    op.create_table(
        "workflow_execution",
        sa.Column("execution_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("executor_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.Column("workflow_version_id", sa.String(length=40), nullable=True),
        sa.Column("workflow_mode_id", clowm.db.types.SQLUUID(length=16), nullable=True),
        sa.Column("debug_path", sa.String(length=1024), nullable=True),
        sa.Column("logs_path", sa.String(length=1024), nullable=True),
        sa.Column("provenance_path", sa.String(length=1024), nullable=True),
        sa.Column("start_time", sa.Integer(), nullable=False),
        sa.Column("end_time", sa.Integer(), nullable=True),
        sa.Column("cpu_hours", sa.Float(), nullable=False),
        sa.Column(
            "status", mysql.ENUM("PENDING", "SCHEDULED", "RUNNING", "CANCELED", "SUCCESS", "ERROR"), nullable=False
        ),
        sa.Column("notes", mysql.TEXT(), nullable=True),
        sa.Column("slurm_job_id", sa.Integer(), nullable=False),
        sa.Column("created_at", sa.Integer(), nullable=False),
        sa.Column("updated_at", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["executor_id"], ["user.uid"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["workflow_mode_id"], ["workflow_mode.mode_id"], ondelete="SET NULL"),
        sa.ForeignKeyConstraint(["workflow_version_id"], ["workflow_version.git_commit_hash"], ondelete="SET NULL"),
        sa.PrimaryKeyConstraint("execution_id"),
    )
    op.create_index(op.f("ix_workflow_execution_execution_id"), "workflow_execution", ["execution_id"], unique=True)
    op.create_table(
        "workflow_modus_to_version_mapping",
        sa.Column("workflow_version_commit_hash", sa.String(length=40), nullable=False),
        sa.Column("workflow_mode_id", clowm.db.types.SQLUUID(length=16), nullable=False),
        sa.ForeignKeyConstraint(["workflow_mode_id"], ["workflow_mode.mode_id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(
            ["workflow_version_commit_hash"], ["workflow_version.git_commit_hash"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("workflow_version_commit_hash", "workflow_mode_id"),
    )
    current_time = int(time.time())
    # Insert roles into database
    op.get_bind().execute(
        sa.sql.text(
            f"""
    INSERT INTO role (role_id, name, created_at, updated_at) VALUES
    (0x{uuid7().hex.upper()}, 'administrator', {current_time}, {current_time}),
    (0x{uuid7().hex.upper()}, 'user', {current_time}, {current_time}),
    (0x{uuid7().hex.upper()}, 'reviewer', {current_time}, {current_time}),
    (0x{uuid7().hex.upper()}, 'developer', {current_time}, {current_time}),
    (0x{uuid7().hex.upper()}, 'db_maintainer', {current_time}, {current_time})
    """
        )
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("workflow_modus_to_version_mapping")
    op.drop_index(op.f("ix_workflow_execution_execution_id"), table_name="workflow_execution")
    op.drop_table("workflow_execution")
    op.drop_index(op.f("ix_workflow_version_git_commit_hash"), table_name="workflow_version")
    op.drop_table("workflow_version")
    op.drop_index(op.f("ix_resource_version_resource_version_id"), table_name="resource_version")
    op.drop_table("resource_version")
    op.drop_table("bucketpermission")
    op.drop_index(op.f("ix_workflow_workflow_id"), table_name="workflow")
    op.drop_table("workflow")
    op.drop_table("user_role_mapping")
    op.drop_index(op.f("ix_resource_resource_id"), table_name="resource")
    op.drop_table("resource")
    op.drop_index(op.f("ix_bucket_name"), table_name="bucket")
    op.drop_table("bucket")
    op.drop_index(op.f("ix_workflow_mode_mode_id"), table_name="workflow_mode")
    op.drop_table("workflow_mode")
    op.drop_index(op.f("ix_user_uid"), table_name="user")
    op.drop_index(op.f("ix_user_lifescience_id"), table_name="user")
    op.drop_table("user")
    op.drop_index(op.f("ix_role_role_id"), table_name="role")
    op.drop_table("role")
    # ### end Alembic commands ###
